from to_sentry.config_parser import ToSentryConfigParser

import logging
import os
import raven
import raven.events
import urllib2
import warnings

import gflags 
FLAGS = gflags.FLAGS 

gflags.DEFINE_integer('digits', 8, 'Length of stderr and stdout line numbers.', lower_bound=1)

def usage():
    print "Usage: to_sentry <sentry feed> Subject line ... "


def format_text(handle, text):
    if not text:
        raise StopIteration()
    formatter = handle + ".%%0%dd" % (FLAGS.digits,)
    for x, line in enumerate(text.split('\n')):
        line = line.rstrip()
        key = formatter % (x,)
        if line:
            yield key, line


def send(channel, message, stdout, stderr=None, client_factory=raven.Client, extra=None):
    if not extra:
        extra = {}
    if hasattr(stdout, 'read'):
        stdout = stdout.read()
    if stderr and hasattr(stderr, 'read'):
        stderr = stderr.read()
    logger = logging.getLogger("sentry.errors")
    handler = logging.StreamHandler()
    formatter = logging.Formatter(("[%(levelname)s] "
                                   "%(name)s: "
                                   "%(message)s\n\nOriginal "
                                   "application "
                                   "error "
                                   "follows:\n") + stdout)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        client = client_factory(dsn=ToSentryConfigParser()[channel])
        if stdout or stderr:
            for key, value in format_text('stdout', stdout):
                extra[key] = value
            for key, value in format_text('stderr', stderr):
                extra[key] = value 
            for key, value in os.environ.items():
                extra["env." + key] = value
            client.capture('Message',
                           message=message,
                           data = None,
                           extra = extra)


def send_old(argv, stdin, client_factory=raven.Client, extra=None):
    return send(argv[0], ' '.join(argv[1:]), stdin, extra=extra)



from ConfigParser import ConfigParser, Error


import gflags 
FLAGS = gflags.FLAGS 

gflags.DEFINE_string('config', '/etc/to_sentry.conf', 
                     'Location of the sentry configuration file.')

class ToSentryConfigParser(ConfigParser):


    def __init__(self, *args, **kwargs):
        ConfigParser.__init__(self, *args, **kwargs)
        if not self.read(FLAGS.config):
            raise IOError('%r not readable.' % (FLAGS.config,))
        
    def __getitem__(self, key):
        try:
            return self.get(key, 'url')
        except Error:
            if key.startswith('http://') or key.startswith('https://'):
                return key
            else:
                raise Error(key)
    



import StringIO
from subprocess import PIPE, Popen
import to_sentry
import to_sentry.transmitter
import gflags
import sys
FLAGS = gflags.FLAGS

gflags.DEFINE_string('message', None, 'A description of what broke')
gflags.DEFINE_string('channel', 'cron', 'Channel name from the config file')
gflags.DEFINE_bool('version', False, 'Show the current version number')
gflags.DEFINE_string('version_check', None, 'Test if the version number is at least this large.  If blank, returns version number')


def main(argv, stdin):
    try:
        if len(argv) <= 1:
            print 'Usage: %s ARGS\\n%s' % (sys.argv[0], FLAGS)
            return 1
        argv = FLAGS(argv)[1:]
    except gflags.FlagsError, e:
        print '%s\\nUsage: %s ARGS\\n%s' % (e, sys.argv[0], FLAGS)
        return 1
    if FLAGS.version or FLAGS.version_check:
        if FLAGS.version:
            print "to_sentry:  %d.%d.%d" % to_sentry.VERSION
        if FLAGS.version_check:
            version = tuple(map(int, FLAGS.version_check.split('.')))
            return not(version <= to_sentry.VERSION)
        else:
            return 0
    elif not FLAGS.message:
        to_sentry.transmitter.send_old(list(argv), stdin, extra={'deprecation warning': 
                                                                 'to_sentry warning: to_sentry <channel> <message> is deprecated.'})
    elif argv:
        p = Popen(argv, stdout=PIPE, stderr=PIPE)
        stdout, stderr = p.communicate()
        returncode = p.returncode
        to_sentry.transmitter.send(FLAGS.channel, FLAGS.message, stdout, stderr, extra={'returncode': str(p.returncode)})
    else:
        to_sentry.transmitter.send(FLAGS.channel, FLAGS.message, stdin)

